' SED_PBWIN - Use the PBWIN compiler
#COMPILE EXE
#DIM ALL
#INCLUDE ONCE "GLFW.INC"
#INCLUDE ONCE "GDIPUTILS.INC"

' =======================================================================================
' Main
' =======================================================================================
FUNCTION PBMAIN () AS LONG

   LOCAL hr, nWidth, nHeight, running, frames, x, y AS LONG
   LOCAL t, t0, fps AS DOUBLE
   LOCAL szTitlestr AS ASCIIZ * 200
   LOCAL xrot, yrot AS SINGLE
   LOCAL strTextureData AS STRING
   LOCAL TextureWidth, TextureHeight AS LONG
   DIM   glHandles(2) AS DWORD
   DIM   LightAmbient(3) AS SINGLE
   DIM   LightDiffuse(3) AS SINGLE
   DIM   LightPosition(3) AS SINGLE
   LOCAL z AS SINGLE
   LOCAL filter AS LONG
   LOCAL xspeed, yspeed AS SINGLE
   LOCAL lp, fp, light, bp, blend AS LONG

   ' Initialize GLFW
   glfwInit

   ' Open OpenGL window
   IF ISFALSE glfwOpenWindow(640, 480, 0, 0, 0, 0, 16, 0, %GLFW_WINDOW) THEN
      glfwTerminate
      EXIT FUNCTION
   END IF

   ' Load bitmap texture from disk
   hr = GdiPlusLoadTexture("glass.bmp", TextureWidth, TextureHeight, strTextureData, %TRUE)

   ' Assign an OpenGL handle to this texture
   glGenTextures 3, glHandles(0)

   ' Activate our newly created texture
   glEnable %GL_TEXTURE_2D

   ' Create Nearest Filtered Texture
   glBindTexture %GL_TEXTURE_2D, glHandles(0)
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MAG_FILTER, %GL_NEAREST
   glTexParameteri %GL_TEXTURE_2D, %GL_TEXTURE_MIN_FILTER, %GL_NEAREST
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Create Linear Filtered Texture
   glBindTexture %GL_TEXTURE_2D, glHandles(1)
   glTexParameteri %GL_TEXTURE_2D,%GL_TEXTURE_MAG_FILTER,%GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D,%GL_TEXTURE_MIN_FILTER,%GL_LINEAR
   glTexImage2D %GL_TEXTURE_2D, 0, 3, TextureWidth, TextureHeight, 0, _
                %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   ' Create MipMapped Texture
   glBindTexture %GL_TEXTURE_2D, glHandles(2)
   glTexParameteri %GL_TEXTURE_2D,%GL_TEXTURE_MAG_FILTER,%GL_LINEAR
   glTexParameteri %GL_TEXTURE_2D,%GL_TEXTURE_MIN_FILTER,%GL_LINEAR_MIPMAP_NEAREST
   gluBuild2DMipmaps %GL_TEXTURE_2D, 3, TextureWidth, TextureHeight, _
                 %GL_RGBA, %GL_UNSIGNED_BYTE, BYVAL STRPTR(strTextureData)

   glEnable %GL_TEXTURE_2D

   ' Specify clear values for the color buffers
   glClearColor 0.0!, 0.0!, 0.0!, 0.0!
   ' Specify the clear value for the depth buffer
   glClearDepth 1.0!
   ' Specify the value used for depth-buffer comparisons
   glDepthFunc %GL_LESS
   ' Enable depth comparisons and update the depth buffer
   glEnable %GL_DEPTH_TEST
   ' Select smooth shading
   glShadeModel %GL_SMOOTH

   LightAmbient(0) = 0.5 : LightAmbient(1) = 0.5 : LightAmbient(2) = 0.5 : LightAmbient(3) = 1.0
   LightDiffuse(0) = 1.0 : LightDiffuse(1) = 1.0 : LightDiffuse(2) = 1.0 : LightDiffuse(3) = 1.0
   LightPosition(0) = 0.0 : LightPosition(1) = 0.0 : LightPosition(2) = 2.0 : LightPosition(3) = 1.0

   glLightfv %GL_LIGHT1, %GL_AMBIENT, LightAmbient(0)
   glLightfv %GL_LIGHT1, %GL_DIFFUSE, LightDiffuse(0)
   glLightfv %GL_LIGHT1, %GL_POSITION, LightPosition(0)
   glEnable %GL_LIGHT1

   glColor4f 1.0!, 1.0!, 1.0!, 0.5!
   glBlendFunc %GL_SRC_ALPHA,%GL_ONE

   ' Enable sticky keys
   glfwEnable %GLFW_STICKY_KEYS

   ' Disable vertical sync (on cards that support it)
   glfwSwapInterval 0

   ' Main loop
   running = %TRUE
   frames = 0
   t0 = glfwGetTime

   z = -5.0

   DO WHILE running

      ' Get time and mouse position
      t = glfwGetTime
      glfwGetMousePos x, y
      ' Calculate and display FPS (frames per second)
      IF t - t0 > 1.0! OR frames = 0 THEN
         fps = frames / (t-t0)
         wsprintf szTitlestr, "NEHE Lesson 08 (%i FPS)", BYVAL fps
         glfwSetWindowTitle szTitlestr
         t0 = t
         frames = 0
      END IF
      frames = frames + 1

      '  Get window size (may be different than the requested size)
      glfwGetWindowSize nWidth, nHeight
      IF nHeight <= 0 THEN nHeight = 1

      ' Set viewport
      glViewport 0, 0, nWidth, nHeight

      ' Clear the screen and depth buffers
      glClearColor 0.0!, 0.0!, 0.0!, 0.0!
      glClear %GL_COLOR_BUFFER_BIT OR %GL_DEPTH_BUFFER_BIT

      ' Select and setup the projection matrix
      glMatrixMode %GL_PROJECTION
      glLoadIdentity

      ' Calculate the aspect ratio of the window
      gluPerspective 45.0!, nWidth/nHeight, 0.1!, 100.0!

      ' Select the modelview matrix
      glMatrixMode %GL_MODELVIEW
      glLoadIdentity

      glTranslatef 0.0!, 0.0!, z

      glRotatef xrot, 1.0!, 0.0!, 0.0!
      glRotatef yrot, 0.0!, 1.0!, 0.0!

      glBindTexture %GL_TEXTURE_2D, glHandles(filter)

      glBegin %GL_QUADS
         ' Front Face
         glNormal3f   0.0!, 0.0!, 1.0
         glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!
         glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!
         glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!,  1.0!
         glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!,  1.0!
         ' Back Face
         glNormal3f   0.0!, 0.0!, -1.0!
         glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!, -1.0!
         glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!
         glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!
         glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!, -1.0!
         ' Top Face
         glNormal3f   0.0!, 1.0!, 0.0!
         glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!
         glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!,  1.0!,  1.0!
         glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!,  1.0!,  1.0!
         glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!
         ' Bottom Face
         glNormal3f   0.0!,-1.0!, 0.0!
         glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!, -1.0!, -1.0!
         glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!, -1.0!, -1.0!
         glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!
         glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!
         ' Right face
         glNormal3f   1.0!, 0.0!, 0.0!
         glTexCoord2f 1.0!, 0.0! : glVertex3f  1.0!, -1.0!, -1.0!
         glTexCoord2f 1.0!, 1.0! : glVertex3f  1.0!,  1.0!, -1.0!
         glTexCoord2f 0.0!, 1.0! : glVertex3f  1.0!,  1.0!,  1.0!
         glTexCoord2f 0.0!, 0.0! : glVertex3f  1.0!, -1.0!,  1.0!
         ' Left Face
         glNormal3f  -1.0!,  0.0!, 0.0!
         glTexCoord2f 0.0!, 0.0! : glVertex3f -1.0!, -1.0!, -1.0!
         glTexCoord2f 1.0!, 0.0! : glVertex3f -1.0!, -1.0!,  1.0!
         glTexCoord2f 1.0!, 1.0! : glVertex3f -1.0!,  1.0!,  1.0!
         glTexCoord2f 0.0!, 1.0! : glVertex3f -1.0!,  1.0!, -1.0!
      glEnd

      xrot = xrot + xspeed
      yrot = yrot + yspeed

      ' Swap buffers
      glfwSwapBuffers

      ' L key
      IF glfwGetKey(76) = %GLFW_PRESS AND ISFALSE lp THEN
         lp = %TRUE
         light = NOT light
         IF ISFALSE light THEN
            glDisable %GL_LIGHTING
         ELSE
            glEnable %GL_LIGHTING
         END IF
      END IF
      IF glfwGetKey(76) <> %GLFW_PRESS THEN lp = %FALSE

      ' F key
      IF glfwGetKey(70) = %GLFW_PRESS AND ISFALSE fp THEN
         fp = %TRUE
         filter = filter + 1
         IF filter > 2 THEN filter = 0
      END IF
      IF glfwGetKey(70) <> %GLFW_PRESS THEN fp = %FALSE

      IF glfwGetKey(%GLFW_KEY_PAGEUP) = %GLFW_PRESS THEN z = z - 0.02!
      IF glfwGetKey(%GLFW_KEY_PAGEDOWN) = %GLFW_PRESS THEN z = z + 0.02!
      IF glfwGetKey(%GLFW_KEY_UP) = %GLFW_PRESS THEN xspeed = xspeed - 0.01!
      IF glfwGetKey(%GLFW_KEY_DOWN) = %GLFW_PRESS THEN xspeed = xspeed + 0.01!
      IF glfwGetKey(%GLFW_KEY_RIGHT) = %GLFW_PRESS THEN yspeed = yspeed + 0.01!
      IF glfwGetKey(%GLFW_KEY_LEFT) = %GLFW_PRESS THEN yspeed = yspeed - 0.01!

      ' B key
      IF glfwGetKey(66) = %GLFW_PRESS AND ISFALSE bp THEN
         bp = %TRUE
         blend = NOT blend
         IF blend THEN
            glEnable %GL_BLEND
            glDisable %GL_DEPTH_TEST
         ELSE
            glDisable %GL_BLEND
            glEnable %GL_DEPTH_TEST
         END IF
      END IF
      IF glfwGetKey(66) <> %GLFW_PRESS THEN bp = %FALSE

      ' Check if the ESC key was pressed or the window was closed
      running = NOT glfwGetKey(%GLFW_KEY_ESC) AND glfwGetWindowParam(%GLFW_OPENED)

   LOOP

   ' Delete the texture
   glDeleteTextures(3, glHandles(0))

   ' Close OpenGL window and terminate GLFW
   glfwTerminate

END FUNCTION
' =======================================================================================
